# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.drivers.evolis_spec import Driver as DriverEvolis


class Driver(DriverEvolis):
    SPEC = {
        "code": "eurobio_i2p",
        "label": "I2P EUROBIO ASTM",
    }

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            physician = [p for p in patient.physicians if p.type.lower() == 'prescriber']
            seg_p = message.patients.add(
                practice_pid=patient.patient_id,
                patient_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                attending_physician=physician[0].code if physician else "",
            )
            for sample, action in patient.samples:
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        collection_date=sample.collection_date,
                        # il faut envoyer S pour Sample
                        specimen_descriptor=sample.specimen,
                        report_type="X",
                    )
                    t = seg_o.tests.add()
                    t.code = order.code
                    seg_p.laboratory_pid = sample.sample_id

        if message.patients or context["query"]:
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.specimen_id.strip(),
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if (
                        not seg_p.practice_pid and
                        not seg_p.laboratory_pid and
                        seg_o.instrument_specimen_id.startswith(tuple(self.reference_qc.split(";")))
                    ):
                        results = worklist.get_or_create(
                            sample_id=seg_o.instrument_specimen_id,
                            control_lot=seg_o.instrument_specimen_id,
                            type="control"
                        ).results
                    else:
                        results = worklist.get_or_create(
                            sample_id=seg_p.laboratory_pid.strip(),
                            type="patient"
                        ).results
                    for seg_r in seg_o.results:
                        lots_reactifs = []
                        for seg_c in seg_r.commentaires:
                            lots_reactifs.append({
                                "name": seg_c.reactif.test_reactif,
                                "lot": seg_c.reactif.lot_reactif,
                                "expiration": seg_c.reactif.date_peremption,
                                "type": "reagent",
                            })
                        results.add(
                            code=seg_r.test_name,
                            value=seg_r.data,
                            flags=seg_r.flags,
                            date=seg_r.date or message.date,
                            lots=lots_reactifs,
                        )
        return worklist
