# -*- coding: utf -*-

from __future__ import unicode_literals
import datetime
import time

from driverng.protocol.mllp import MLLPProtocol


class AutoplakMLLPProtocolHL7(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s|P|2.5\rMSA|AA|%(ack_id)s|\r"""

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted = payload.splitlines()[0].split(b"|")
        date = date or datetime.datetime.now()
        type_message = splitted[8]
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"control_id": control_id,
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"ack_id": splitted[9],
        }
        if self.is_automate:
            context[b"message_type"] = b"OML^O33^OML_O33"
        else:
            if type_message.endswith(b"OUL_R22"):
                context[b"message_type"] = b"ACK^R22^ACK"
                return AutoplakMLLPProtocolHL7.HL7_ACK % context

    def send_trames(self, trames):
        self.write(trames)
        entetes = self.get_payload(trames).splitlines()[0].split(b"|")
        if not entetes[8].endswith(b"RSP_K11"):  # queries
            self.recv_ack()
        return True

    def recv_trames(self):
        data = self.mllp_readline()
        payload = self.get_payload(data)
        splitted = payload.splitlines()[0].split(b"|")
        type_message = splitted[8]
        if type_message.endswith(b"OUL_R22"):
            ack_message = self.get_ack_message(payload, control_id=splitted[9])
            if ack_message:
                self.write(self.build_trames(ack_message))
        return payload
